@@ -50,8 +50,8 @@ class TestNVMeCreateMaxNS(TestNVMe):
         TestNVMe.__init__(self)
         self.dps = 0
         self.flbas = 0
-        self.nsze = 0x1400000
-        self.ncap = 0x1400000
+        self.nsze = self.get_ncap() / self.get_format() / self.get_max_ns()
+        self.ncap = self.nsze
         self.setup_log_dir(self.__class__.__name__)
         self.max_ns = self.get_max_ns()
         self.ctrl_id = self.get_ctrl_id()
